package com.xylink.util;

import com.xylink.config.SDKConfigMgr;
import com.xylink.log.XyLinkLog;
import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.RandomStringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.KeyFactory;
import java.security.Security;
import java.security.spec.X509EncodedKeySpec;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.Base64;
import java.util.regex.Pattern;

public class AuthcodeUtil {


    public static final String SEPARATOR = ",";
    // private static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
    public static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
    public static Pattern extUserIdMatcher = Pattern.compile("^[0-9a-zA-Z_-]{0,50}$");
    public static final String KEY_ALGORITHM = "RSA";
    private static XyLinkLog logger = SDKConfigMgr.getXyLinkLog();
    public static final String ALGORITHM = "SM4/CBC/PKCS5Padding";

    static {
        Security.addProvider(new BouncyCastleProvider());
    }

    /**
     * RSA加密方式获取authcode
     * @param externalUserId
     * @param pubKey
     * @return authCode usable in nowtime +/- 2 minutes （避免云机器时间差过大）
     * @throws Exception
     */
    public static String generateAuthCodeByRSA(String externalUserId, String pubKey) {
        if (!matchExtUserId(externalUserId)){
            logger.info(externalUserId + " externalUserId formatter error");
            return null;
        }
        String ramdom = RandomStringUtils.random(11, true, false);
        LocalDateTime dateTime = LocalDateTime.now();
        String timeStr = dateTime.format(timeStrFormatter);
        String content = externalUserId + SEPARATOR + timeStr+ SEPARATOR+ramdom;
        logger.info("加密前："+content);
        return encryptByPubKey4String(content, pubKey);
    }

    public static byte[] rsaEncryptByPublicKey(byte[] data, String publicKey)
            throws Exception {
        //得到公钥
        byte[] keyBytes = Base64.getDecoder().decode(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        // 对数据加密
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicK);
        return cipher.doFinal(data);
    }

    public static byte[] encryptByPubKey4Byte(String data, String publicKey) {
        if (data == null || data.equals("")) {
            return null;
        }
        byte[] publicEncrypData = null;
        try {
            //1、data base64 encode
            String dataBase64Encoded = Base64.getEncoder().encodeToString(data.getBytes(StandardCharsets.UTF_8));
            logger.info("rsaencrypt dataBase64Encoded "+dataBase64Encoded);
            //2、encrypt
            publicEncrypData = rsaEncryptByPublicKey(dataBase64Encoded.getBytes(StandardCharsets.UTF_8), publicKey);
        } catch (Exception e) {
            logger.error("rsaEncryptByPubKey4Byte error "+e);
        }
        return publicEncrypData;
    }

    public static String encryptByPubKey4String(String data, String publicKey) {
        byte[] bytes = encryptByPubKey4Byte(data, publicKey);
        return Hex.encodeHexString(bytes);
    }

    public static boolean matchExtUserId(String extUserId){
        logger.info(extUserId.length()+" extUserId ->"+extUserId);
        return extUserIdMatcher.matcher(extUserId).find();
    }


    public static String SM4Encrypt(String key, String content){
        try {
            Cipher instance = Cipher.getInstance(ALGORITHM, BouncyCastleProvider.PROVIDER_NAME);
            byte[] bytes = key.getBytes(StandardCharsets.UTF_8);
            byte[] keyBytes = getBytes(bytes);
            SecretKeySpec secretKey = new SecretKeySpec(keyBytes, ALGORITHM);
            Arrays.sort(keyBytes);
            IvParameterSpec ivParameterSpec = new IvParameterSpec(keyBytes);
            instance.init(Cipher.ENCRYPT_MODE, secretKey, ivParameterSpec);
            byte[] contentBytes = content.getBytes(StandardCharsets.UTF_8);
            byte[] bytes1 = instance.doFinal(contentBytes, 0, contentBytes.length);
            return org.bouncycastle.util.encoders.Hex.toHexString(bytes1);
        } catch (Exception e) {
            logger.error(" ",e);
            return null;
        }
    }

    public static String SM4DeEnt(String key, String content){
        try {
            Cipher instance = Cipher.getInstance(ALGORITHM, BouncyCastleProvider.PROVIDER_NAME);
            byte[] bytes = key.getBytes(StandardCharsets.UTF_8);
            byte[] keyBytes = getBytes(bytes);
            SecretKeySpec secretKey = new SecretKeySpec(keyBytes, ALGORITHM);
            Arrays.sort(keyBytes);
            IvParameterSpec ivParameterSpec = new IvParameterSpec(keyBytes);
            instance.init(Cipher.DECRYPT_MODE, secretKey, ivParameterSpec);
            byte[] decode = org.bouncycastle.util.encoders.Hex.decode(content);
            byte[] bytes1 = instance.doFinal(decode, 0, decode.length);
            return new String(bytes1);
        } catch (Exception e) {
            logger.error(" ",e);
            return null;
        }
    }

    private static byte[] getBytes(byte[] keyBytes) {
        byte[] reslutBytes = new byte[16];
        System.arraycopy(keyBytes, 0, reslutBytes, 0, Math.min(keyBytes.length, 16));
        return reslutBytes;
    }

    public static String generateAuthCodeBySM4(String externalUserId, String key) {
        String random =  RandomStringUtils.random(10,true,false);
        String content = externalUserId +";"+ System.currentTimeMillis() + ";"+ random;
        return SM4Encrypt(key, content);
    }

    public static void main(String[] args) throws Exception {
        String externalUserId = "luodantest1";
        String key = "4ao0QdOb";
        String s = generateAuthCodeBySM4(externalUserId, key);
        String value = SM4DeEnt(key, s);
        System.out.println(value);
    }

}
