package com.xylink.util;

import com.xylink.config.SDKConfigMgr;
import com.xylink.log.XyLinkLog;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.util.*;

public class SignUtil {

    private static XyLinkLog logger = SDKConfigMgr.getXyLinkLog();

    public static final String SIGN_SIGN_KEY_NEW = "x-xy-sign";
    public static final String SIGN_SIGN_TYPE_KEY_NEW = "x-xy-signtype";
    public static final String Authorization = "Authorization";
    public static final String SIGN_CLIENT_ID_KEY_NEW = "x-xy-clientid";
    public static final String SIGN_NONCE_KEY_NEW = "x-xy-nonce";
    public static final String SIGN_TIMESTAMP_KEY_NEW = "x-xy-timestamp";
    public static final String X_XY_SIGNATUREVERSION = "x-xy-signatureversion";

    public static String signSecret = "XX";
    public static String x_xy_clientid = "XX";
    public static String access_token = "XX";
    public static String x_xy_signtype = SignType.HMAC_SHA256.name();

    /**
     * test
     *
     * @param args
     */
    public static void main(String[] args) {
        String method = "POST";
        String uri = "/api/rest/external/v1/create_meeting?enterpriseId=12f52fdahj2cd5c46825e5b7926d23663b9";
        String requestBody = "{\"meetingName\": \"my first cloudRoom\"}";
        Map<String, String> headersMap = new HashMap<String, String>(16);
        headersMap.put("x-xy-nonce", RandomStringUtils.random(60, true, false));
        headersMap.put("x-xy-timestamp", String.valueOf(System.currentTimeMillis()));
        headersMap.put("x-xy-clientid", x_xy_clientid);
        headersMap.put("x-xy-signtype", x_xy_signtype);

        // 签名计算
        String sign = SignUtil.getSign(method, headersMap, uri, requestBody, signSecret);
        logger.error("最终签名：" + sign);
        //System.out.println("最终签名：" + sign);
        headersMap.put("Authorization", "Bearer " + access_token);
        headersMap.put(SIGN_SIGN_KEY_NEW, sign);
        logger.error("最终header：" + headersMap);
        //System.out.println("最终header：" + headersMap);
    }

    /**
     * 计算签名
     *
     * @param method
     * @param headersMap
     * @param uri
     * @param requestBody
     * @param signSecret    加密密钥
     * @return
     */
    public static String getSign(String method, Map<String, String> headersMap, String uri, String requestBody, String signSecret) {
        requestBody = StringUtils.isBlank(requestBody)?"":requestBody;
        byte[] requestBodyBytes = requestBody.getBytes(Charset.forName("UTF-8"));
        if (headersMap == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        //1.添加method
        sb.append(method).append("\n");
        //header排序
        Set<String> keySet = headersMap.keySet();
        String[] keyArray = keySet.toArray(new String[keySet.size()]);
        Arrays.sort(keyArray);
        //2.添加Headers
        for (String k : keyArray) {
            if (k.equals(SIGN_SIGN_KEY_NEW)) {
                continue;
            }
            Object param = headersMap.get(k);
            String paramStr = String.valueOf(param);
            if (paramStr.trim().length() > 0) {
                // 参数值为空，则不参与签名
                sb.append(k).append("=").append(paramStr.trim()).append("&");
            }
        }
        sb = sb.deleteCharAt(sb.length() - 1);//去除最后一个&
        sb.append("\n");
        //3.添加uri
        sb.append(uri);
        //4.添加请求的body
        sb.append("\n");
        if (requestBodyBytes != null && requestBodyBytes.length > 0) {
            sb.append(DigestUtils.md5Hex(requestBodyBytes));
        } else {
            sb.append(DigestUtils.md5Hex("".getBytes(Charset.forName("UTF-8"))));
        }
        //5.添加密钥（signSecret + "&"）
        sb.append("\n");
        sb.append(signSecret+"&");
//        logger.error("======================sign string "+ SDKConfigMgr.getSignatureVersion().getVersion() +"=============================");
        logger.info(sb.toString());
//        logger.error("======================sign string "+ SDKConfigMgr.getSignatureVersion().getVersion() +"=============================");
        //System.out.println("======================sign string "+ SDKConfigMgr.getSignatureVersion().getVersion() +"=============================");
        //System.out.println(sb.toString());
        //System.out.println("======================sign string "+ SDKConfigMgr.getSignatureVersion().getVersion() +"=============================");
        String signStr = "";
        //加密
        SignType type = null;
        String signType = headersMap.get(SIGN_SIGN_TYPE_KEY_NEW);
        if (signType != null && signType.length() > 0) {
            type = SignType.valueOf(signType);
        }
        if (type == null) {
            type = SignType.MD5;
        }
        switch (type) {
            case MD5:
                signStr = DigestUtils.md5Hex(sb.toString()).toUpperCase();
                break;
            case SHA256:
                signStr = DigestUtils.sha256Hex(sb.toString()).toUpperCase();
                break;
            case HMAC_SHA256:
                signStr = sha256HMAC(sb.toString(), signSecret+"&").toUpperCase();
                break;
            default:
                break;
        }
        return signStr;
    }

    /**
     * sha256_HMAC加密
     *
     * @param message 消息
     * @param secret  秘钥
     * @return 加密后字符串
     */
    public static String sha256HMAC(String message, String secret) {
        String hash = "";
        try {
            Mac sha256_HMAC = Mac.getInstance("HmacSHA256");
            SecretKeySpec secret_key = new SecretKeySpec(secret.getBytes("UTF-8"), "HmacSHA256");
            sha256_HMAC.init(secret_key);
            byte[] bytes = sha256_HMAC.doFinal(message.getBytes("UTF-8"));
            hash = byteArrayToHexString(bytes);
        } catch (Exception e) {
        }
        return hash;
    }

    /**
     * 将加密后的字节数组转换成字符串
     *
     * @param b 字节数组
     * @return 字符串
     */
    public static String byteArrayToHexString(byte[] b) {
        StringBuilder hs = new StringBuilder();
        String stmp;
        for (int n = 0; b != null && n < b.length; n++) {
            stmp = Integer.toHexString(b[n] & 0XFF);
            if (stmp.length() == 1) {
                hs.append('0');
            }
            hs.append(stmp);
        }
        return hs.toString().toLowerCase();
    }

    public static String randomAlphanumeric(int count) {
        return random(count, 0, 0, true, true,null, new Random());
    }

    public static String random(int count, int start, int end, boolean letters, boolean numbers,
                                char[] chars, Random random) {
        if (count == 0) {
            return "";
        } else if (count < 0) {
            throw new IllegalArgumentException("Requested random string length " + count + " is less than 0.");
        }
        if ((start == 0) && (end == 0)) {
            end = 'z' + 1;
            start = ' ';
            if (!letters && !numbers) {
                start = 0;
                end = Integer.MAX_VALUE;
            }
        }

        char[] buffer = new char[count];
        int gap = end - start;

        while (count-- != 0) {
            char ch;
            if (chars == null) {
                ch = (char) (random.nextInt(gap) + start);
            } else {
                ch = chars[random.nextInt(gap) + start];
            }
            if ((letters && Character.isLetter(ch))
                    || (numbers && Character.isDigit(ch))
                    || (!letters && !numbers))
            {
                if(ch >= 56320 && ch <= 57343) {
                    if(count == 0) {
                        count++;
                    } else {
                        // low surrogate, insert high surrogate after putting it in
                        buffer[count] = ch;
                        count--;
                        buffer[count] = (char) (55296 + random.nextInt(128));
                    }
                } else if(ch >= 55296 && ch <= 56191) {
                    if(count == 0) {
                        count++;
                    } else {
                        // high surrogate, insert low surrogate before putting it in
                        buffer[count] = (char) (56320 + random.nextInt(128));
                        count--;
                        buffer[count] = ch;
                    }
                } else if(ch >= 56192 && ch <= 56319) {
                    // private high surrogate, no effing clue, so skip it
                    count++;
                } else {
                    buffer[count] = ch;
                }
            } else {
                count++;
            }
        }
        return new String(buffer);
    }

    public enum SignType {
        MD5, SHA256, HMAC_SHA256;

        public static boolean contains(String type) {
            for (SignType typeEnum : SignType.values()) {
                if (typeEnum.name().equals(type)) {
                    return true;
                }
            }
            return false;
        }
    }
}
