package com.xylink.demo;

import com.xylink.config.EnterpriseSdkConfig;
import com.xylink.config.SDKConfigMgr;
import com.xylink.config.SignatureVersion;
import com.xylink.log.XyLinkLogType;
import com.xylink.model.MeetingStatus;
import com.xylink.model.SdkMeeting;
import com.xylink.sdk.SdkMethodV2;
import com.xylink.sdk.conferenceControl.ConferenceControlApi;
import com.xylink.util.Result;
import com.xylink.util.SignUtil;
import org.junit.Test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Classname CreateMeetingRoom_sign2_0_MultiEnterprise
 * @Date 2023/10/31 11:38
 * @Created by anan
 * 多企业调用
 * 签名2.0及以上需要的参数
 * 1、apiHost
 * 2、clientId
 * 3、clientSecret
 * 4、enterpriseId

 * 初始化方式为：
 * SDKConfigMgr.initSDKConfig();
 *
 * 流程和CreateMeetingRoom_sign2_0中基本一致，请参考
 */
public class CreateMeetingRoom_sign2_0_MultiEnterprise {
    //企业1信息
    String apiHost1 = "https://sdkapi.xylink.com";
    String clientId1 = "";
    String clientSecret1 = "";
    String enterpriseId1 = "";

    //企业2信息
    String apiHost2 = "";
    String clientId2 = "";
    String clientSecret2 = "";
    String enterpriseId2 = "";

    //企业3信息
    String apiHost3 = "";
    String clientId3 = "";
    String clientSecret3 = "";
    String enterpriseId3 = "";

    private void initConfig() {
        //初始化多个企业
        //环境信息配置2
        EnterpriseSdkConfig sdkConfig = new EnterpriseSdkConfig();
        sdkConfig.setEnterpriseId(enterpriseId1);
        sdkConfig.setClientId(clientId1);
        sdkConfig.setClientSecret(clientSecret1);
        sdkConfig.setServerHost(apiHost1);
        //以上为基础配置，和单企业一样，我们可以在基础配置的基础添加额外配置
        sdkConfig.setSignType(SignUtil.SignType.MD5);
        sdkConfig.setSignatureVersion(SignatureVersion.Signature_2_0);
        sdkConfig.setEnableSecurityHttp(false);
        sdkConfig.setExtrasHeader(null);

        //环境信息配置1
        EnterpriseSdkConfig sdkConfig1 = new EnterpriseSdkConfig();
        sdkConfig1.setEnterpriseId(enterpriseId2);
        sdkConfig1.setClientId(clientId2);
        sdkConfig1.setClientSecret(clientSecret2);
        sdkConfig1.setServerHost(apiHost2);
        //以上为基础配置，和单企业一样，我们可以在基础配置的基础添加额外配置
        sdkConfig1.setSignType(SignUtil.SignType.MD5);
        sdkConfig1.setSignatureVersion(SignatureVersion.Signature_2_0);
        sdkConfig1.setEnableSecurityHttp(false);
        sdkConfig1.setExtrasHeader(null);

        //环境信息配置3
        EnterpriseSdkConfig sdkConfig2 = new EnterpriseSdkConfig();
        sdkConfig2.setEnterpriseId(enterpriseId3);
        sdkConfig2.setClientId(clientId3);
        sdkConfig2.setClientSecret(clientSecret3);
        sdkConfig2.setServerHost(apiHost3);
        //以上为基础配置，和单企业一样，我们可以在基础配置的基础添加额外配置
        sdkConfig2.setSignType(SignUtil.SignType.MD5);
        sdkConfig2.setSignatureVersion(SignatureVersion.Signature_2_0);
        sdkConfig2.setEnableSecurityHttp(false);
        sdkConfig2.setExtrasHeader(null);

        List<EnterpriseSdkConfig> sdkConfigList = new ArrayList<EnterpriseSdkConfig>();
        sdkConfigList.add(sdkConfig);
        sdkConfigList.add(sdkConfig1);
        sdkConfigList.add(sdkConfig2);
        SDKConfigMgr.initSDKConfig(sdkConfigList, XyLinkLogType.Log4j);
    }

    @Test
    public void testMethod2_0_MultiEnterprise() throws IOException {
        //初始化配置
        initConfig();

        //至此，jar包已经初始化完成，下面将是我们的调用环节, 签名2.0请使用SdkMethodV2，不要使用SdkMethodV1
        //此处我们列举了创建会议室接口示例
        //多企业只用在接口中切换企业即可调用不同企业的接口

        //enterpriseId1 创建会议室
        String url = "/api/rest/external/v1/create_meeting?enterpriseId=" + enterpriseId1;
        HashMap<String, Object> body = new HashMap<>();
        body.put("meetingName", "sdk测试1");
        Result<SdkMeeting> result1 = SdkMethodV2.sdkPost(url, body, SdkMeeting.class);
        if (result1.isSuccess()) {
            System.out.println("result: " + result1);
        } else {
            System.out.println("result: " + result1);
            throw new RuntimeException("faild to Create Meeting Room");
        }

        //enterpriseId2 创建会议室
        String url2 = "/api/rest/external/v1/create_meeting?enterpriseId=" + enterpriseId2;
        HashMap<String, Object> body2 = new HashMap<>();
        body.put("meetingName", "sdk测试2");
        Result<SdkMeeting> result2 = SdkMethodV2.sdkPost(url2, body2, SdkMeeting.class);
        if (result2.isSuccess()) {
            System.out.println("result: " + result2);
        } else {
            System.out.println("result: " + result2);
            throw new RuntimeException("faild to Create Meeting Room");
        }

    }
}
